//  gsavarr
//  save / rest array to plain ascii file
// reads the file in memory and writes the whole file, for fast result
// by - Anand K Gupta  Fri  11-10-96

*   SaveArray() .......  syntax to be written
*                       
*

//  In the following macros cT is a text string

// ---------- my definations
#define FWeol(cT)       cT += chr(13)+chr(10)
#define FReol(cT)       cT := iif(subs(cT,1,1)==chr(13),subs(cT,2),cT); ;
                        cT := iif(subs(cT,1,1)==chr(10),subs(cT,2),cT)

// Write NIL element: U<5 digit length><character value>
#define SAVE_NIL(cT)    cT += "U" + "NIL"; FWeol(cT)

// Read a NIL element
#define GET_NIL(cT)     left(cT, 3); cT := subs(cT, 3+1); FReol(cT)


// Write character element: C<5 digit length><character value>
#define SAVE_CHAR(cT, cv)   cT += "C" + Str(Len(cv), 5) + cv; FWeol(cT)

// Write date element: D<datestring>
#define SAVE_DATE(cT, dv)   cT += "D" + Dtoc(dv); FWeol(cT)

// Write logical element: L<T or F character>
#define SAVE_LOG(cT, lv)    cT += "L" + Iif(lv, "T", "F"); FWeol(cT)

// Write numeric element: N<2 digit length> <numeric formatted as a string>
#define SAVE_NUM(cT, nv)    cT += "N" + Str(Len(Str(nv)),2) + Str(nv); FWeol(cT)

// Write array header: A<4 digit array length>
#define WRITE_AHD(cT, av)   cT += "A" + Str(Len(av), 4); FWeol(cT)

// Read a type
#define GET_TYPE(cT)        left(cT, 1); cT := subs(cT, 1+1)

// Read an array size
#define GET_AR_SIZE(cT)     Val(left(cT, 4)); cT := subs(cT, 4+1); FReol(cT)

// Read a character element
#define GET_CHAR(cT, n)     left(cT, n); cT := subs(cT, n+1); FReol(cT)

// Read a date element
#define GET_DATE(cT)        Ctod(left(cT, 10)); cT := subs(cT, 10+1); FReol(cT)

// Read a logical element
#define GET_LOG(cT)         (left(cT, 1) == "T"); cT := subs(cT, 1+1); FReol(cT)

// Read a numeric element
#define GET_NUM(cT, n)       Val(left(cT, n)); cT := subs(cT, n+1); FReol(cT)


/***
*   SaveArray(aArray, cFileName)
*
*    aArray[]    -Array to save
*    cFileName   -File in which to save array
*/
FUNCTION SaveArray(aArray, cFileName)
    LOCAL fHandle := FCreate(cFileName), cText

    cText := ''
    SaveArr(fHandle, aArray, @cText)
//debuglog(2, aArray, cText)
    // now write the whole text to file
    FWrite(fHandle, cText)
    FClose(fHandle)

RETURN NIL

/***
*    RestArray(cFileName)
*
*    cFileName   - -File from which to restore array
*/
FUNCTION RestArray(cFileName)
    * LOCAL aRestArr, fHandle := FOpen(cFileName)
    LOCAL aRestArr, cText

    cText := memoread(cFileName)
    GET_TYPE(cText)
    aRestArr := RestArr(cText)
    * FClose(fHandle)

RETURN aRestArr

// create the file in memory and writes the whole file, for fast result
// by - Anand K Gupta  Fri  11-10-96
//
STATIC FUNCTION SaveArr(fHandle, aArray, cText)
    LOCAL LenArray, ElemType, i

    // Write the array header
    WRITE_AHD(cText, aArray)
    LenArray := Len(aArray)

    // Write each element in the array ~EL
    FOR i := 1 TO LenArray
        ElemType := ValType(aArray[i])
        DO CASE
        CASE ElemType == "A"
            // A subarray, recurse ~EL
            SaveArr(cText, aArray[i], @cText)

        CASE ElemType == "C"
            // Save a character element
            SAVE_CHAR(cText, aArray[i])

        CASE ElemType == "D"
            // Save a date element
            SAVE_DATE(cText, aArray[i])

        CASE ElemType == "L"
            // Save a logical element
            SAVE_LOG(cText, aArray[i])

        CASE ElemType == "N"
            // Save a numeric element
            SAVE_NUM(cText, aArray[i])

        OTHERWISE
            // anything else is saved as NIL
            SAVE_NIL(cText)

        ENDCASE
    NEXT

RETURN NIL

// reads the whole file in memory and get array from it, for fast result
// by - Anand K Gupta  Fri  11-10-96
//
STATIC FUNCTION RestArr(cText)
    LOCAL LenArray, i, ElemType, aArray, nLen

    // Determine array's size and allocate it
    LenArray := GET_AR_SIZE(cText)
    aArray   := Array(LenArray)

    // Restore each element
    FOR i := 1 TO LenArray
        // Determine its type
        ElemType := GET_TYPE(cText)
        DO CASE
        CASE ElemType == "A"
            // A sub array so recurse ~EL
            aArray[i] := RestArr(@cText)

        CASE ElemType == "C"
            nLen := val(left(cText, 5))
            cText := subs(cText, 5+1)
            aArray[i] := GET_CHAR(cText, nLen)

        CASE ElemType == "D"
            aArray[i] := GET_DATE(cText)

        CASE ElemType == "L"
            aArray[i] := GET_LOG(cText)

        CASE ElemType == "N"
            nLen := val(left(cText, 2))
            cText := subs(cText, 2+1)
            aArray[i] := GET_NUM(cText, nLen)

        OTHERWISE
            // anything else is restored as NIL
            GET_NIL(cText)

        ENDCASE
    NEXT

RETURN aArray

